@@ -7,7 +7,9 @@ import logging
 import threading
 from typing import (
     Any,
+    Awaitable,
     Callable,
+    cast,
     List,
     Optional,
 )
@@ -20,6 +22,7 @@ from tracers.analyzers import (
 from tracers.constants import (
     LOGGER_DEFAULT,
     LOOP_CHECK_INTERVAL,
+    T,
 )
 from tracers.containers import (
     DaemonResult,
@@ -97,17 +100,17 @@ def trace(  # noqa: MC0001
     enabled: bool = True,
     function_name: str = '',
     log_to: Optional[logging.Logger] = LOGGER_DEFAULT,
-) -> Callable[[Callable[..., Any]], Callable[..., Any]]:
+) -> Callable[[Callable[..., T]], Callable[..., T]]:
 
-    def decorator(function: Callable[..., Any]) -> Callable[..., Any]:
+    def decorator(function: Callable[..., T]) -> Callable[..., T]:
 
         if asyncio.iscoroutinefunction(function):
 
             @functools.wraps(function)
-            async def isolated_wrapper(*args: Any, **kwargs: Any) -> Any:
+            async def isolated_wrapper(*args: Any, **kwargs: Any) -> T:
 
                 @functools.wraps(function)
-                async def wrapper(*args: Any, **kwargs: Any) -> Any:
+                async def wrapper(*args: Any, **kwargs: Any) -> T:
                     if LEVEL.get() == 0:
                         LOGGER.set(log_to)
 
@@ -123,7 +126,9 @@ def trace(  # noqa: MC0001
                                 )
 
                             record_event('call', function, function_name)
-                            result = await function(*args, **kwargs)
+                            result = await cast(
+                                Awaitable[T], function(*args, **kwargs),
+                            )
                             record_event('return', function, function_name)
 
                             if LEVEL.get() == 1:
@@ -140,19 +145,21 @@ def trace(  # noqa: MC0001
                         TRACING.set(False)
 
                         # No overhead is introduced!
-                        result = await function(*args, **kwargs)
+                        result = await cast(
+                            Awaitable[T], function(*args, **kwargs),
+                        )
 
                     return result
 
-                return await wrapper(*args, **kwargs)
+                return cast(T, await wrapper(*args, **kwargs))
 
         elif callable(function):
 
             @functools.wraps(function)
-            def isolated_wrapper(*args: Any, **kwargs: Any) -> Any:
+            def isolated_wrapper(*args: Any, **kwargs: Any) -> T:
 
                 @functools.wraps(function)
-                def wrapper(*args: Any, **kwargs: Any) -> Any:
+                def wrapper(*args: Any, **kwargs: Any) -> T:
                     if LEVEL.get() == 0:
                         LOGGER.set(log_to)
 
@@ -180,7 +187,7 @@ def trace(  # noqa: MC0001
                     return result
 
                 if LEVEL.get() == 0:
-                    result = contextvars.copy_context().run(
+                    result: T = contextvars.copy_context().run(
                         wrapper, *args, **kwargs
                     )
                 else:
@@ -201,8 +208,8 @@ def trace(  # noqa: MC0001
     return decorator
 
 
-_DEFAULT_TRACER: Callable[[Callable[..., Any]], Callable[..., Any]] = trace()
+_DEFAULT_TRACER: Callable[[Callable[..., T]], Callable[..., T]] = trace()
 
 
-def call(function: Callable[..., Any], *args: Any, **kwargs: Any) -> Any:
+def call(function: Callable[..., T], *args: Any, **kwargs: Any) -> T:
     return _DEFAULT_TRACER(function)(*args, **kwargs)
